;
; Checks the documentation for various errors

#Region Includes
#include "Array.au3"
#include "FileConstants.au3"
#include "include\OutputLib.au3"
#include "include\SettingsLib.au3"
#EndRegion Includes

#Region Global Variables
; The name of the project.
Global Const $g_sProject = "Check libExamples .au3"
Global Const $g_sProjectDir = "docs\autoit\english\txt2htm\libExamples"

#EndRegion Global Variables

#Region Main body of code
Global $g_nDiff = 0, $g_nIgnored = 0
Global $g_nExitCode = Main($g_sProject)
Exit $g_nExitCode
#EndRegion Main body of code

#Region Main()
; ===================================================================
; Main()
;
; The main program body.
; Parameters:
;	None.
; Returns:
;	None.
; ===================================================================
Func Main($sProject)
	Local $nReturn = 0

	; Create the output window and initial message.
	_OutputWindowCreate()
	_OutputProgressWrite("==== Output for " & StringTrimRight(@ScriptName, StringLen(".exe")) & " (" & $sProject & ") ====" & @CRLF)

	; Set the build directory based on the rules and the INI file value.
	_SettingBuildDir()

	$nReturn = Verify_SameLibExamples($g_sProjectDir)
	ToolTip("")

	; Check the return value.
	If $nReturn Then
		_OutputProgressWrite("Checking failed (" & $nReturn & ")." & @CRLF)
	Else
		If $g_nDiff Then
			_OutputProgressWrite("(" & $g_nIgnored & "," & $g_nDiff & ")Checking complete." & @CRLF)
		Else
			_OutputProgressWrite("Checking complete." & @CRLF)
		EndIf
	EndIf

	; Write closing message and wait for close (if applicable).
	_OutputProgressWrite("Finished." & @CRLF & @CRLF) ; Two CRLF's in case of chained output.
	_OutputWaitClosed($nReturn)

	; Return the value.
	Return $nReturn
EndFunc   ;==>Main
#EndRegion Main()

#Region Primary Functions

#Region Verify_SameLibExamples()

Func Verify_SameLibExamples($sProjectDir)
	_OutputProgressWrite("LibExamples verification ..." & @CRLF)
	Local $sFile, $sSetFile, $sExample, $sSetExample, $nReturn = 0
	Local $hSearch = FileFindFirstFile($sProjectDir & "\*.au3")
	While 1
		$sFile = FileFindNextFile($hSearch)
		If @error Then ExitLoop
		ToolTip($sFile, 200, 200)

		If StringInStr($sFile, "_Get") = 0 Then ContinueLoop

		$sSetFile = StringReplace($sFile, "_Get", "_Set")
		$sSetExample = FileRead($sProjectDir & "\" & $sSetFile)
		If @error Then
			$g_nIgnored += 1
			ContinueLoop
		EndIf

		$sExample = FileRead($sProjectDir & "\" & $sFile)
		If $sExample == $sSetExample Then
			$nReturn += 1
			_OutputBuildWrite($nReturn & " Examples are identical " & $sFile & @CRLF)
		Else
			$g_nDiff += 1
			DisplayDiff($sProjectDir, $sFile, $sSetFile)
			_OutputProgressWrite($g_nDiff & " Examples are different " & $sFile & @CRLF)

;~ 			If $g_nDiff = 1 Then
;~ 			DisplayDiff($sProjectDir, $sFile, $sSetFile)
			;_OutputWaitClosed($nReturn)
;~ 				Exit
;~ 			EndIf
		EndIf

	WEnd

	; Close the search handle
	FileClose($hSearch)

	Return $nReturn
EndFunc   ;==>Verify_SameLibExamples

#EndRegion Verify_SameLibExamples()

#Region DisplayDiff()

Func DisplayDiff($sDir, $sFile, $sSetFile)
	Run("explorer.exe " & $sFile, $sDir)
	Run("explorer.exe " & $sSetFile, $sDir)
	RunWait(StringReplace(@ProgramFilesDir, " (x86)", "") & "\WinMerge\WinMergeU.exe " & $sFile & " " & $sSetFile, $sDir)

EndFunc   ;==>DisplayDiff

#EndRegion DisplayDiff()

#EndRegion Primary Functions
