; ==============================================================================
; Generate the "ExitCodes.htm" page and also include " AutoItExitCode.au3"
; from the "AutoIt.rc" and "resource.h"

; Output directories are read from 'txt2htm.ini'
;
; 02 Jun 2020 - jpm - jpm @ autoitscript.com
; ==============================================================================

#include <FileConstants.au3>
#include <StringConstants.au3>

#include "..\..\_build\include\MiscLib.au3"
#include "..\..\_build\include\OutputLib.au3"

Opt("TrayIconDebug", 1)

OnAutoItExitRegister("OnQuit") ; ### when run in standalone
_OutputWindowCreate() ; ### when run in standalone

FileChangeDir(@ScriptDir)

Global $g_sAPPENDIX_OUT_DIR = IniRead("txt2htm.ini", "Output", "Appendix", "ERR")
Global $g_sRESOURCE_IN_DIR = IniRead("txt2htm.ini", "Input", "resources", "ERR")

If $g_sAPPENDIX_OUT_DIR = "ERR" Then
	MsgBox($MB_SYSTEMMODAL, "Error", "Could not read txt2htm.ini")
	Exit
EndIf

Local $sExitCodesHTM = $g_sAPPENDIX_OUT_DIR & "ExitCodes.htm"
_OutputProgressWrite("ExitCodes.htm updating ...")

; get Map from "AutoIt.rc"
Local $sAutoItRC = $g_sRESOURCE_IN_DIR & "AutoIt.rc"
Global $g_aAutoItRc = Extract_IDS_AUT_E($sAutoItRC)

; get array from "resource.h"
Local $sResourceH = $g_sRESOURCE_IN_DIR & "resource.h"
Global $g_aResourceH = Extract_IDS_Index($sResourceH)

; create a temporary new "ExitCodes.htm"
Local $sTempHTM = @TempDir & "\ExitCodes.htm"
Global $g_hOut = FileOpen($sTempHTM, BitOR($FO_OVERWRITE, $FO_UTF8))

; get header from previous "ExitCodes.htm"
Local $hIn = FileOpen($sExitCodesHTM)
Local $sLine
While 1
	$sLine = FileReadLine($hIn)
	put($sLine)
	If StringInStr($sLine, '<th>Fatal Error message</th>') Then ExitLoop
WEnd
put('    </tr>')

FileClose($hIn)

; create all Exit codes table Entries
For $i = 1 To $g_aResourceH[0][0]
	put('    <tr>')
	put('      <td>0x7FFFF' & Hex($g_aResourceH[$i][0], 3) & '</td>')
	put('      <td>' & Get_IDS_AUT_E_String($g_aResourceH[$i][1]) & '</td>')
	put('    </tr>')
Next

put('  </table>')
put('</body>')
put('</html>')

FileClose($g_hOut)

; Update "ExitCodes.htm"
If FileRead($sTempHTM) <> FileRead($sExitCodesHTM) Then
	_OutputProgressWrite("Updated ")
	FileMove($sTempHTM, $sExitCodesHTM, $FC_OVERWRITE)
Else
	FileDelete($sTempHTM)
EndIf

_OutputProgressWrite("Complete." & @CRLF)

; ===============================================================================================================================
Global $g_sINCLUDE_OUT_DIR = IniRead("txt2htm.ini", "Output", "Includes", "ERR")

Local $sExitCodesAU3 = $g_sINCLUDE_OUT_DIR & "AutoItFatalExitConstants.au3"
_OutputProgressWrite("AutoItFatalExitConstants.au3 updating ...")

; create a temporary new "AutoItExitCodes.au3"
Local $sTempAU3 = @TempDir & "\AutoiItExitCodes.au3"
$g_hOut = FileOpen($sTempAU3, BitOr($FO_OVERWRITE, $FO_ANSI))

; get header from previous "AutoItExitCodes.au3"
$hIn = FileOpen($sExitCodesAU3)
While 1
	$sLine = FileReadLine($hIn)
	If StringInStr($sLine, '$EXITFATALCODES') Then ExitLoop
	put($sLine)
WEnd
$sLine = StringRegExpReplace($sLine, "(\[\d*\])", "[" & $g_aResourceH[0][0] & "]", 1)
put($sLine)

FileClose($hIn)

; create all Exit codes table Entries
Local $sCont = ", _"
For $i = 1 To $g_aResourceH[0][0]
	If $i = $g_aResourceH[0][0] Then $sCont = " _"
	put(@TAB & @TAB & '[0x7FFFF' & Hex($g_aResourceH[$i][0], 3) & ", '" & Get_IDS_AUT_E_String($g_aResourceH[$i][1], "'") & "']" & $sCont)
Next
put(@TAB & @TAB & ']')
put('; ===============================================================================================================================')

FileClose($g_hOut)

; Update "AutoItExitCodes.au3"
If FileRead($sTempAU3) <> FileRead($sExitCodesAU3) Then
	_OutputProgressWrite(@TAB & "Updated" & @TAB)
	FileMove($sTempAU3, $sExitCodesAU3, $FC_OVERWRITE)
Else
	FileDelete($sTempAU3)
EndIf

_OutputProgressWrite("Complete." & @CRLF)

Exit

Func OnQuit()
	_OutputWaitClosed() ; ### when run in standalone
EndFunc   ;==>OnQuit

Func Extract_IDS_AUT_E($sFile)
	Local $sLine, $aRet[200][2], $n = 0, $i
	Local $hIn = FileOpen($sFile)

	While 1
		$sLine = FileReadLine($hIn)
		If @error Then ExitLoop
		$sLine = StringStripWS($sLine, $STR_STRIPLEADING + $STR_STRIPTRAILING)
		If StringLeft($sLine, 10) = 'IDS_AUT_E_' Then
			$i = StringInStr($sLine, '"')
			If $i = 0 Then
				$sLine &= FileReadLine($hIn)
				$i = StringInStr($sLine, '"')
			EndIf
			$n += 1
			$aRet[$n][0] = StringStripWS(StringLeft($sLine, $i - 1), $STR_STRIPTRAILING)
			$sLine = StringMid($sLine, $i)
			$sLine = StringTrimRight(StringTrimLeft($sLine, 1), 1)
			$sLine = StringReplace($sLine, '""', '"')

			; go around typo in AutoIt generation
			$sLine = StringReplace($sLine, 'Wend', 'WEnd')
			$sLine = StringReplace($sLine, '"or ', '" or ')
			$sLine = StringReplace($sLine, ' Or ', ' or ')

			$aRet[$n][1] = $sLine
		EndIf
	WEnd

	FileClose($sFile)

	$aRet[0][0] = $n
	ReDim $aRet[$n + 1][2]
	Return $aRet
EndFunc   ;==>Extract_IDS_AUT_E

Func Get_IDS_AUT_E_String($sIDS, $sProtect = "")
	Local $sRet
	For $i = 1 To $g_aAutoItRc[0][0]
		If $g_aAutoItRc[$i][0] = $sIDS Then
			$sRet = $g_aAutoItRc[$i][1]
			If $sProtect Then $sRet = StringReplace($sRet, $sProtect, "''")
			Return $sRet
		EndIf
	Next
	Return '???'
EndFunc   ;==>Get_IDS_AUT_E_String

Func Extract_IDS_Index($sFile)
	Local $sLine, $aRet[200][2], $n = 0, $i, $aTemp
	Local $hIn = FileOpen($sFile)

	While 1
		$sLine = FileReadLine($hIn)
		If @error Then ExitLoop
		If $sLine = '' Then ExitLoop
		If StringLeft($sLine, 2) = '//' Then ContinueLoop
		$sLine = StringReplace($sLine, @TAB, ' ')
		$sLine = StringStripWS($sLine, $STR_STRIPSPACES)
		$aTemp = StringSplit($sLine, " ")
		$i = Int($aTemp[3])
		If $i < 104 Then ContinueLoop
		$n += 1
		$aRet[$n][0] = $i
		$aRet[$n][1] = $aTemp[2]
	WEnd

	FileClose($sFile)

	$aRet[0][0] = $n
	ReDim $aRet[$n + 1][2]
	Return $aRet
EndFunc   ;==>Extract_IDS_Index

; ------------------------------------------------------------------------------
; Write a new line to the output file
; ------------------------------------------------------------------------------
Func put($sLineToWrite)
	FileWriteLine($g_hOut, $sLineToWrite)
EndFunc   ;==>put
